/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Channel Variables
 *
 * \author Mark Spencer <markster@digium.com> 
 */

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 89425 $")

#include "asterisk/chanvars.h"
#include "asterisk/strings.h"
#include "asterisk/utils.h"
#include "asterisk/options.h"

struct ast_var_t *ast_var_assign(const char *name, const char *value)
{	
	struct ast_var_t *var;
	int name_len = strlen(name) + 1;
	int value_len = strlen(value) + 1;

	if (!(var = ast_calloc(sizeof(*var) + name_len + value_len, sizeof(char)))) {
		return NULL;
	}

	ast_copy_string(var->name, name, name_len);
	var->value = var->name + name_len;
	ast_copy_string(var->value, value, value_len);
	
	return var;
}	
	
void ast_var_delete(struct ast_var_t *var)
{
	if (var)
		ast_free(var);
}

const char *ast_var_name(const struct ast_var_t *var)
{
	const char *name;

	if (var == NULL || (name = var->name) == NULL)
		return NULL;
	/* Return the name without the initial underscores */
	if (name[0] == '_') {
		name++;
		if (name[0] == '_')
			name++;
	}
	return name;
}

const char *ast_var_full_name(const struct ast_var_t *var)
{
	return (var ? var->name : NULL);
}

const char *ast_var_value(const struct ast_var_t *var)
{
	return (var ? var->value : NULL);
}

int ast_var_copy(struct varshead *to, const struct varshead *from)
{
        struct ast_var_t *current, *newvar;
        const char *varname;
        int count = 0;

        AST_LIST_TRAVERSE(from, current, entries) {
                int vartype = 0;

                varname = ast_var_full_name(current);
                if (!varname)
                        continue;

                if (varname[0] == '_') {
                        vartype = 1;
                        if (varname[1] == '_')
                                vartype = 2;
                }

                switch (vartype) {
                case 1:
                        newvar = ast_var_assign(&varname[1], ast_var_value(current));
                        if (newvar) {
                                AST_LIST_INSERT_TAIL(to, newvar, entries);
                                if (option_debug)
                                        ast_log(LOG_DEBUG, "Copying soft-transferable variable %s.\n", ast_var_name(newvar));

				count++;
                        }
                        break;
                case 2:
                        newvar = ast_var_assign(ast_var_full_name(current), ast_var_value(current));
                        if (newvar) {
                                AST_LIST_INSERT_TAIL(to, newvar, entries);
                                if (option_debug)
                                        ast_log(LOG_DEBUG, "Copying hard-transferable variable %s.\n", ast_var_name(newvar));

				count++;
                        }
                        break;
                default:
                        if (option_debug)
                                ast_log(LOG_DEBUG, "Not copying variable %s.\n", ast_var_name(current));
                        break;
                }
        }

	return count;
}

